// ----------------------------------------------------
// Made by: Peepss
// Warning! Do not use this code without my permission.
// ----------------------------------------------------

import settings from "./Settings.js";

import {
    world,
    system,
    ItemStack
} from "@minecraft/server";
import { ActionFormData } from "@minecraft/server-ui";
import backpackEventsManager from "./customEvents/BackpackEventsManager.js";

world.afterEvents.worldInitialize.subscribe(() => {
    world.gameRules.showTags = settings.showTags;
});

backpackEventsManager.event("onHoldBackpack", ({ player, item }) => {
    const playerInventory = player.getComponent("inventory").container;
    const selectedSlotItem = playerInventory.getItem(player.selectedSlotIndex);

    if (selectedSlotItem) {
        item.lockMode = "slot";
        playerInventory.setItem(player.selectedSlotIndex, item);

        const backpackEntity = backpackEventsManager.spawnBackpackEntity(player, item);

        backpackEventsManager.loadInventory(player, backpackEntity, item);
    }

    //Play sound effects.
    player.playSound("bundle.insert", { pitch: 1.0, volume: 1.0 });
});

backpackEventsManager.event("onReleaseBackpack", ({ player, item }) => {
    const { dimension } = player;

    const playerInventory = player.getComponent("minecraft:inventory").container;
    const itemData = JSON.parse(item.getDynamicProperty("peepss:item_data"));

    dimension.getEntities({
        type: "peepss:backpack_container"
    }).forEach((entity) => {
        const backpackData = JSON.parse(entity.getDynamicProperty("peepss:backpack_data"));

        if (backpackData.backpackId == itemData.backpackId) {
            backpackEventsManager.saveInventory(player, entity, item);
            entity.triggerEvent("minecraft:despawn");
        }
    })

    for (let slot = 0; slot < playerInventory.size; slot++) {
        const currentItem = playerInventory.getItem(slot);

        if (currentItem) {
            let parsedCurrentItemData = null;
            const currentItemData = currentItem.getDynamicProperty("peepss:item_data");
            if (currentItemData) {
                try {
                    parsedCurrentItemData = JSON.parse(currentItemData);
                } catch (e) {
                    console.warn(`Erro ao analisar os dados do item no slot ${slot}:`, e);
                }
            }

            const currentBackpackId = parsedCurrentItemData?.backpackId || null;

            if (currentBackpackId === itemData.backpackId || settings.prohibitedItems.includes(currentItem.typeId) || !currentItemData) {
                currentItem.lockMode = "none";
                playerInventory.setItem(slot, currentItem);
            }
        }
    }

    //Play sound effects.
    player.playSound("bundle.remove_one", { pitch: 1.0, volume: 1.0 });
});


backpackEventsManager.event("onBackpackInventoryOpen", ({ player, item }) => {
    const playerInventory = player.getComponent("minecraft:inventory").container;

    const itemData = item.getDynamicProperty("peepss:item_data");
    const parsedItemData = itemData ? JSON.parse(itemData) : null;
    const itemBackpackId = parsedItemData ? parsedItemData.backpackId : null;

    for (let slot = 0; slot < playerInventory.size; slot++) {
        const currentItem = playerInventory.getItem(slot);

        if (currentItem) {
            const currentItemData = currentItem.getDynamicProperty("peepss:item_data");
            const parsedCurrentItemData = currentItemData ? JSON.parse(currentItemData) : null;
            const currentBackpackId = parsedCurrentItemData ? parsedCurrentItemData.backpackId : null;

            if (settings.prohibitedItems.includes(currentItem.typeId)) {
                if (currentItemData) {
                    if (currentBackpackId !== itemBackpackId) {
                        currentItem.lockMode = "inventory";
                        playerInventory.setItem(slot, currentItem);
                    }
                } else {
                    currentItem.lockMode = "inventory";
                    playerInventory.setItem(slot, currentItem);
                }
            }
        }
    }
});

backpackEventsManager.event("onBackpackInventoryClose", ({ player, item }) => {
    const playerInventory = player.getComponent("minecraft:inventory").container;

    const itemData = item.getDynamicProperty("peepss:item_data");
    const parsedItemData = itemData ? JSON.parse(itemData) : null;
    const itemBackpackId = parsedItemData ? parsedItemData.backpackId : null;

    for (let slot = 0; slot < playerInventory.size; slot++) {
        const currentItem = playerInventory.getItem(slot);

        if (currentItem) {
            const currentItemData = currentItem.getDynamicProperty("peepss:item_data");
            const parsedCurrentItemData = currentItemData ? JSON.parse(currentItemData) : null;
            const currentBackpackId = parsedCurrentItemData ? parsedCurrentItemData.backpackId : null;

            if (settings.prohibitedItems.includes(currentItem.typeId)) {
                if (currentItemData) {
                    if (currentBackpackId !== itemBackpackId) {
                        currentItem.lockMode = "none";
                        playerInventory.setItem(slot, currentItem);
                    }
                } else {
                    currentItem.lockMode = "none";
                    playerInventory.setItem(slot, currentItem);
                }
            }
        }
    }
});

// ---------------------------------
// World Events
// ---------------------------------

//When player leaves in multiplayer when holding a backpack, despawn the backpack container.
world.beforeEvents.playerLeave.subscribe(({ player }) => {
    const { dimension } = player;

    try {
        dimension.getEntities().forEach((entity) => {
            const backpackData = JSON.parse(entity.getDynamicProperty("peepss:backpack_data"));

            if (backpackData.owner == player.name) {
                system.run(() => {
                    entity.triggerEvent("minecraft:despawn");
                })
            }
        })
    } catch { }
});

//When player leaves and joins the world again in singleplayer when holding a backpack, despawn the backpack container.
world.afterEvents.playerSpawn.subscribe(({ player }) => {
    const { dimension } = player;

    try {
        dimension.getEntities().forEach((entity) => {
            const backpackData = JSON.parse(entity.getDynamicProperty("peepss:backpack_data"));

            if (backpackData.owner == player.name) {
                system.run(() => {
                    entity.triggerEvent("minecraft:despawn");
                })
            }
        })
    } catch { }
});

function addItemData(source, pocketsType, currentItem, slot) {
    if (!currentItem) return;

    const currentItemData = currentItem.getDynamicProperty("peepss:item_data");

    if (currentItemData) return;

    if (settings.backpacksTypeIds.includes(currentItem.typeId)) {
        const backpackId = Math.floor(Math.random() * 9000) + 1000;

        let itemDataInterface = {
            owner: source.name,
            backpackId: backpackId,
            typeId: currentItem.typeId,
            slotsAmount: 27,
            isBeingHeldByPlayer: false,
            pockets: {
                leatherPockets: false,
                goldenPockets: false,
                endlessPockets: false,
            },
        };

        switch (pocketsType) {
            case "leather_pockets":
                itemDataInterface.pockets.leatherPockets = true;
                break;
            case "golden_pockets":
                itemDataInterface.pockets.goldenPockets = true;
                break;
            case "endless_pockets":
                itemDataInterface.pockets.endlessPockets = true;
                break;
        }

        currentItem.setDynamicProperty("peepss:item_data", JSON.stringify(itemDataInterface));

        const playerInventory = source.getComponent("minecraft:inventory").container;
        playerInventory.setItem(slot, currentItem);
    }
}

function changeItemData(source, pocketsType, currentItem, slot) {
    if (!currentItem) return;

    const currentItemData = currentItem.getDynamicProperty("peepss:item_data");

    if (!currentItemData) return; // Sai apenas se o item NÃO tiver dados

    if (settings.backpacksTypeIds.includes(currentItem.typeId)) {
        const parsedCurrentItemData = JSON.parse(currentItemData);

        // Atualizando os dados existentes do item
        switch (pocketsType) {
            case "leather_pockets":
                parsedCurrentItemData.pockets.leatherPockets = true;
                break;
            case "golden_pockets":
                parsedCurrentItemData.pockets.goldenPockets = true;
                break;
            case "endless_pockets":
                parsedCurrentItemData.pockets.endlessPockets = true;
                break;
        }

        // Salvando os dados atualizados de volta no item
        currentItem.setDynamicProperty("peepss:item_data", JSON.stringify(parsedCurrentItemData));

        // Atualizando o item no inventário do jogador
        const playerInventory = source.getComponent("minecraft:inventory").container;
        playerInventory.setItem(slot, currentItem);
    }
}

function showPocketsScreen(source, pocketsType) {
    const playerInventory = source.getComponent("minecraft:inventory").container;
    const screen = new ActionFormData();

    screen.title("screen.pockets_upgrade");
    screen.body("description.pockets_upgrade");

    const buttonActions = [];

    for (let slot = 0; slot < playerInventory.size; slot++) {
        const currentItem = playerInventory.getItem(slot);

        if (currentItem) {
            if (settings.backpacksTypeIds.includes(currentItem.typeId)) {
                const itemData = currentItem.getDynamicProperty("peepss:item_data");
                const nameTag = currentItem.nameTag || `item.${currentItem.typeId}`;

                if (itemData) {
                    const parsedItemData = JSON.parse(itemData);
                    if (
                        !parsedItemData.pockets.leatherPockets &&
                        !parsedItemData.pockets.goldenPockets &&
                        !parsedItemData.pockets.endlessPockets
                    ) {
                        screen.button(nameTag, `textures/items/${currentItem.typeId.replace("peepss:", "")}`);
                        buttonActions.push(() => {
                            changeItemData(source, pocketsType, currentItem, slot);
                            playerInventory.setItem(source.selectedSlotIndex, null);
                        });
                    } else {
                        screen.button("button.locked", `textures/items/${currentItem.typeId.replace("peepss:", "")}`);
                        buttonActions.push(() => { });
                    }
                } else {
                    screen.button(nameTag, `textures/items/${currentItem.typeId.replace("peepss:", "")}`);
                    buttonActions.push(() => {
                        addItemData(source, pocketsType, currentItem, slot);
                        playerInventory.setItem(source.selectedSlotIndex, null);
                    });
                }
            } else {
                screen.button("button.locked");
                buttonActions.push(() => { });
            }
        } else {
            screen.button("button.locked");
            buttonActions.push(() => { });
        }
    }

    screen.show(source).then(response => {
        if (response.canceled) return;

        const clickedIndex = response.selection;
        if (clickedIndex !== undefined && buttonActions[clickedIndex]) {
            buttonActions[clickedIndex]();
        }
    });
}

world.afterEvents.itemStartUse.subscribe((eventData) => {
    const { source, itemStack } = eventData;

    if (settings.pocketsTypeIds.includes(itemStack.typeId)) {
        const pocketsType = itemStack.typeId.replace("peepss:", "");
        showPocketsScreen(source, pocketsType);
    }
});